@@ -78,7 +78,14 @@ sub add_symbol {
 
     $key = join '-', $symbol->{'pcode'}, $symbol->{'code'};
 
-    #carp "$key exists!" if($FROM_CODE{$key});
+    if($FROM_CODE{$key}) {
+        confess(sprintf 'Key collision (%s): (%s) tries to overwrite (%s)',
+            $key,
+            (join ',', map { $symbol->{$_} } @OBJECT_ATTRIBUTES),
+            (join ',', map { $FROM_CODE{$key}->{$_} } @OBJECT_ATTRIBUTES),
+        );
+    }
+
     $FROM_CODE{$key} = $symbol;
     push @{ $FROM_ID{ $symbol->{'id'} } }, $symbol;
 
@@ -311,8 +318,8 @@ See L<DOCSIS::ConfigFile>
 #        ID                     CODE PCODE   FUNC         L_LIMIT   H_LIMIT     LENGTH
 #        identifier      docsis_code   pID   func         low_limit high_limit  length
 #-------------------------------------------------------------------------------------
-__PACKAGE__->add_symbol([ map { $_ eq '_' ? undef : $_ } @$_ ]) for(
-    [qw/ Pad                       0     0   _            0         0           1 /],
+__PACKAGE__->add_symbol($_) for(
+    [qw/ Pad                       0     0   nested       0         255         1 /],
     [qw/ DownstreamFrequency       1     0   uint         88000000  860000000   1 /],
     [qw/ UpstreamChannelId         2     0   uchar        0         255         1 /],
     [qw/ CmMic                     6     0   mic          0         0           1 /],
@@ -518,8 +525,7 @@ __PACKAGE__->add_symbol([ map { $_ eq '_' ? undef : $_ } @$_ ]) for(
     [qw/ TftpModemAddress         20     0   ip           0         0           1 /],
 
     # Generic TLV... we only use the limits  code and length dont matter
-    #[qw/ GenericTLV                0     0   nested       0         255         1 /],
-    [qw/ GenericTLV              255     0   nested       0         0           1 /],
+    [qw/ GenericTLV              255     0   no_value     0         0           1 /],
 );
 #-------------------------------------------------------------------------------------
 #        ID                     CODE PCODE   FUNC         L_LIMIT   H_LIMIT     LENGTH
